@@ -1,5 +1,5 @@
 from collections import deque
-from typing import Any, Iterable
+from typing import Any, Iterable, Dict
 
 from plenum.common.constants import BATCH, OP_FIELD_NAME
 from plenum.common.prepare_batch import split_messages_on_batches
@@ -117,8 +117,8 @@ class Batched(MessageProcessor):
                                 timeout=self.messageTimeout,
                                 serialized=True)
                     else:
-                        logger.warning("Cannot create batch(es) for {}".format(
-                            self, dest))
+                        logger.warning("{} cannot create batch(es) for {}"
+                                       .format(self, dest))
                 else:
                     while msgs:
                         msg = msgs.popleft()
@@ -129,7 +129,7 @@ class Batched(MessageProcessor):
                             "{} sending msg {} to {}".format(self, msg, dest))
 
         for rid in removedRemotes:
-            logger.warning("{}{} rid {} has been removed"
+            logger.warning("{}{} has removed rid {}"
                            .format(CONNECTION_PREFIX, self, rid),
                            extra={"cli": False})
             msgs = self.outBoxes[rid]
@@ -141,8 +141,11 @@ class Batched(MessageProcessor):
             del self.outBoxes[rid]
 
     def _make_batch(self, msgs):
-        batch = Batch(msgs, None)
-        serialized_batch = self.sign_and_serialize(batch)
+        if len(msgs) > 1:
+            batch = Batch(msgs, None)
+            serialized_batch = self.sign_and_serialize(batch)
+        else:
+            serialized_batch = msgs[0]
         return serialized_batch
 
     def _test_batch_len(self, batch_len):
