@@ -1,46 +1,47 @@
 import heapq
+import math
 import operator
-from collections import Callable
+import time
+from collections import Callable, Counter
 from functools import partial
 from random import shuffle
 from typing import Any, List, Dict, Tuple
-import math
 from typing import Optional
 
-import time
-from plenum.common.ledger import Ledger
 from ledger.merkle_verifier import MerkleVerifier
 from ledger.util import F
-
-from plenum.common.messages.node_messages import LedgerStatus, CatchupRep, \
-    ConsistencyProof, f, CatchupReq
+from plenum.common.config_util import getConfig
 from plenum.common.constants import POOL_LEDGER_ID, LedgerState, DOMAIN_LEDGER_ID, \
     CONSISTENCY_PROOF, CATCH_UP_PREFIX
-from plenum.common.util import compare_3PC_keys, SortedDict
-from plenum.common.config_util import getConfig
+from plenum.common.ledger import Ledger
+from plenum.common.ledger_info import LedgerInfo
+from plenum.common.messages.node_messages import LedgerStatus, CatchupRep, \
+    ConsistencyProof, f, CatchupReq
+from plenum.common.txn_util import reqToTxn
+from plenum.common.util import compare_3PC_keys, SortedDict, mostCommonElement, min_3PC_key
+from plenum.server.has_action_queue import HasActionQueue
 from plenum.server.quorums import Quorums
 from stp_core.common.constants import CONNECTION_PREFIX
 from stp_core.common.log import getlogger
-from plenum.server.has_action_queue import HasActionQueue
-from plenum.common.ledger_info import LedgerInfo
-from plenum.common.txn_util import reqToTxn
-
 
 logger = getlogger()
 
 
 class LedgerManager(HasActionQueue):
-
     def __init__(self,
                  owner,
-                 ownedByNode: bool=True,
-                 postAllLedgersCaughtUp: Optional[Callable]=None,
-                 preCatchupClbk: Callable = None):
-
+                 ownedByNode: bool = True,
+                 postAllLedgersCaughtUp: Optional[Callable] = None,
+                 preCatchupClbk: Optional[Callable] = None,
+                 ledger_sync_order: Optional[List] = None):
+        # If ledger_sync_order is not provided (is None), it is assumed that
+        # `postCatchupCompleteClbk` of the LedgerInfo will be used
         self.owner = owner
         self.ownedByNode = ownedByNode
         self.postAllLedgersCaughtUp = postAllLedgersCaughtUp
         self.preCatchupClbk = preCatchupClbk
+        self.ledger_sync_order = ledger_sync_order
+
         self.config = getConfig()
         # Needs to schedule actions. The owner of the manager has the
         # responsibility of calling its `_serviceActions` method periodically
@@ -48,7 +49,7 @@ class LedgerManager(HasActionQueue):
 
         # Holds ledgers of different types with
         # their info like callbacks, state, etc
-        self.ledgerRegistry = {}   # type: Dict[int, LedgerInfo]
+        self.ledgerRegistry = {}  # type: Dict[int, LedgerInfo]
 
         # Largest 3 phase key received during catchup.
         # This field is needed to discard any stashed 3PC messages or
@@ -63,11 +64,11 @@ class LedgerManager(HasActionQueue):
         return self._serviceActions()
 
     def addLedger(self, iD: int, ledger: Ledger,
-                  preCatchupStartClbk: Callable=None,
-                  postCatchupStartClbk: Callable=None,
-                  preCatchupCompleteClbk: Callable=None,
-                  postCatchupCompleteClbk: Callable=None,
-                  postTxnAddedToLedgerClbk: Callable=None):
+                  preCatchupStartClbk: Callable = None,
+                  postCatchupStartClbk: Callable = None,
+                  preCatchupCompleteClbk: Callable = None,
+                  postCatchupCompleteClbk: Callable = None,
+                  postTxnAddedToLedgerClbk: Callable = None):
 
         if iD in self.ledgerRegistry:
             logger.error("{} already present in ledgers "
@@ -90,11 +91,14 @@ class LedgerManager(HasActionQueue):
         if ledgerInfo.consistencyProofsTimer is None:
             return
 
+        proofs = ledgerInfo.recvdConsistencyProofs
+        # there is no any received ConsistencyProofs
+        if not proofs:
+            return
         logger.debug("{} requesting consistency "
                      "proofs after timeout".format(self))
 
-        quorum = Quorums(self.owner.totalNodes - 1)
-        proofs = ledgerInfo.recvdConsistencyProofs
+        quorum = Quorums(self.owner.totalNodes)
         groupedProofs, null_proofs_count = self._groupConsistencyProofs(proofs)
         if quorum.same_consistency_proof.is_reached(null_proofs_count):
             return
@@ -143,8 +147,6 @@ class LedgerManager(HasActionQueue):
 
         logger.debug("{} requesting {} missing transactions "
                      "after timeout".format(self, num_missing))
-        # eligibleNodes = list(self.nodestack.conns -
-        #                      self.blacklistedNodes)
         eligibleNodes = self.nodes_to_request_txns_from
 
         if not eligibleNodes:
@@ -216,7 +218,6 @@ class LedgerManager(HasActionQueue):
         numElgNodes = len(eligibleNodes)
         for i, req in enumerate(cReqs):
             nodeName = eligibleNodes[i % numElgNodes]
-            # self.send(req, self.nodestack.getRemote(nodeName).uid)
             self.sendTo(req, nodeName)
 
         ledgerInfo.catchupReplyTimer = time.perf_counter()
@@ -231,11 +232,13 @@ class LedgerManager(HasActionQueue):
         self.getLedgerInfoByType(ledgerType).state = state
 
     def setLedgerCanSync(self, ledgerType: int, canSync: bool):
-        if ledgerType not in self.ledgerRegistry:
+        try:
+            ledger_info = self.getLedgerInfoByType(ledgerType)
+            ledger_info.canSync = canSync
+        except KeyError:
             logger.error("ledger type {} not present in ledgers so "
                          "cannot set state".format(ledgerType))
             return
-        self.getLedgerInfoByType(ledgerType).canSync = canSync
 
     def prepare_ledgers_for_sync(self):
         for ledger_info in self.ledgerRegistry.values():
@@ -254,7 +257,7 @@ class LedgerManager(HasActionQueue):
         ledgerStatus = LedgerStatus(*status)
         if ledgerStatus.txnSeqNo < 0:
             self.discard(status, reason="Received negative sequence number "
-                         "from {}".format(frm),
+                                        "from {}".format(frm),
                          logMethod=logger.warning)
             return
         ledgerId = getattr(status, f.LEDGER_ID.nm)
@@ -305,6 +308,11 @@ class LedgerManager(HasActionQueue):
         # post sending this ledger status
         ledgerInfo.recvdConsistencyProofs[frm] = None
         ledgerInfo.ledgerStatusOk.add(frm)
+
+        if self.isLedgerSame(ledgerStatus):
+            ledgerInfo.last_txn_3PC_key[frm] = \
+                (ledgerStatus.viewNo, ledgerStatus.ppSeqNo)
+
         if self.has_ledger_status_quorum(
                 len(ledgerInfo.ledgerStatusOk), self.owner.totalNodes):
             logger.debug("{} found out from {} that its "
@@ -316,18 +324,33 @@ class LedgerManager(HasActionQueue):
                 # If this node's ledger is same as the ledger status (which is
                 #  also the majority of the pool), then set the last ordered
                 # 3PC key
-                key = (ledgerStatus.viewNo, ledgerStatus.ppSeqNo)
                 self.do_pre_catchup(ledgerId)
-                if self.isLedgerSame(ledgerStatus) and key != (None, None):
-                    # Any state cleaup that is part of pre-catchup should be
-                    # done
-                    self.catchupCompleted(ledgerId, key)
-                else:
-                    self.catchupCompleted(ledgerId)
+                # Any state cleanup that is part of pre-catchup should be
+                # done
+                last_3PC_key = self._get_last_txn_3PC_key(ledgerInfo)
+                self.catchupCompleted(ledgerId, last_3PC_key)
             else:
                 # Ledger was already synced
                 self.mark_ledger_synced(ledgerId)
 
+    def _get_last_txn_3PC_key(self, ledgerInfo):
+        quorum = Quorums(self.owner.totalNodes)
+        quorumed_3PC_keys = \
+            [
+                most_common_element
+                for most_common_element, freq in
+                Counter(ledgerInfo.last_txn_3PC_key.values()).most_common()
+                if quorum.ledger_status_last_3PC.is_reached(freq) and
+                most_common_element[0] is not None and
+                most_common_element[1] is not None
+            ]
+
+        if len(quorumed_3PC_keys) == 0:
+            return None
+
+        min_quorumed_3PC_key = min_3PC_key(quorumed_3PC_keys)
+        return min_quorumed_3PC_key
+
     @staticmethod
     def has_ledger_status_quorum(leger_status_num, total_nodes):
         quorum = Quorums(total_nodes).ledger_status
@@ -396,7 +419,7 @@ class LedgerManager(HasActionQueue):
     def checkLedgerIsOutOfSync(self, ledgerInfo) -> bool:
         recvdConsProof = ledgerInfo.recvdConsistencyProofs
         # Consider an f value when this node had not been added
-        adjustedQuorum = Quorums(self.owner.totalNodes - 1)
+        adjustedQuorum = Quorums(self.owner.totalNodes)
         equal_state_proofs = self.__get_equal_state_proofs_count(
             recvdConsProof)
         return not adjustedQuorum.same_consistency_proof.is_reached(
@@ -571,7 +594,7 @@ class LedgerManager(HasActionQueue):
         # Certain transactions might need to be
         # transformed to certain format before applying to the ledger
         txn = reqToTxn(txn)
-        z = txn if not self.ownedByNode else  \
+        z = txn if not self.ownedByNode else \
             self.owner.transform_txn_for_ledger(txn)
         return z
 
@@ -630,7 +653,7 @@ class LedgerManager(HasActionQueue):
         for k, catchupReps in ledgerInfo.recvdCatchupRepliesFrm.items():
             for rep in catchupReps:
                 txns = getattr(rep, f.TXNS.nm)
-                # Transfers of odcits in RAET converts integer keys to string
+
                 if str(seqNo) in txns:
                     return k, rep
 
@@ -691,7 +714,7 @@ class LedgerManager(HasActionQueue):
         ledgerInfo = self.getLedgerInfoByType(ledgerId)
         recvdConsProof = ledgerInfo.recvdConsistencyProofs
         # Consider an f value when this node was not connected
-        adjustedQuorum = Quorums(self.owner.totalNodes - 1)
+        adjustedQuorum = Quorums(self.owner.totalNodes)
         if len(recvdConsProof) == adjustedQuorum.f + 1:
             # At least once correct node believes that this node is behind.
 
@@ -720,7 +743,7 @@ class LedgerManager(HasActionQueue):
 
         logger.debug(
             "{} cannot start catchup since received only {} "
-            "consistency proofs but need at least {}". format(
+            "consistency proofs but need at least {}".format(
                 self,
                 len(recvdConsProof),
                 adjustedQuorum.consistency_proof.value))
@@ -753,7 +776,7 @@ class LedgerManager(HasActionQueue):
         return recvdPrf, nullProofs
 
     def _reliableProofs(self, groupedProofs):
-        adjustedQuorum = Quorums(self.owner.totalNodes - 1)
+        adjustedQuorum = Quorums(self.owner.totalNodes)
         result = {}
         for (start, end), val in groupedProofs.items():
             for (view_no, lastPpSeqNo, oldRoot,
@@ -844,10 +867,9 @@ class LedgerManager(HasActionQueue):
                             .format(CATCH_UP_PREFIX, self, ledgerId))
 
     def _getCatchupTimeout(self, numRequest, batchSize):
-        return numRequest * (self.config.CatchupTransactionsTimeout +
-                             0.1 * batchSize)
+        return numRequest * self.config.CatchupTransactionsTimeout
 
-    def catchupCompleted(self, ledgerId: int, last_3PC: Tuple=(0, 0)):
+    def catchupCompleted(self, ledgerId: int, last_3PC: Optional[Tuple] = None):
         if ledgerId not in self.ledgerRegistry:
             logger.error("{}{} called catchup completed for ledger {}".
                          format(CATCH_UP_PREFIX, self, ledgerId))
@@ -856,10 +878,12 @@ class LedgerManager(HasActionQueue):
         # Since multiple ledger will be caught up and catchups might happen
         # multiple times for a single ledger, the largest seen
         # ppSeqNo needs to be known.
-        if compare_3PC_keys(self.last_caught_up_3PC, last_3PC) > 0:
+        if last_3PC is not None \
+                and compare_3PC_keys(self.last_caught_up_3PC, last_3PC) > 0:
             self.last_caught_up_3PC = last_3PC
 
         self.mark_ledger_synced(ledgerId)
+        self.catchup_next_ledger(ledgerId)
 
     def mark_ledger_synced(self, ledger_id):
         ledgerInfo = self.getLedgerInfoByType(ledger_id)
@@ -875,6 +899,38 @@ class LedgerManager(HasActionQueue):
                    for l in self.ledgerRegistry.values()):
                 self.postAllLedgersCaughtUp()
 
+    def catchup_next_ledger(self, ledger_id):
+        if not self.ownedByNode:
+            logger.debug('{} not owned by node'.format(self))
+            return
+        next_ledger_id = self.ledger_to_sync_after(ledger_id)
+        if next_ledger_id is not None:
+            self.catchup_ledger(next_ledger_id)
+        else:
+            logger.debug('{} not found any ledger to catchup after {}'
+                         .format(self, ledger_id))
+
+    def catchup_ledger(self, ledger_id):
+        try:
+            ledger_info = self.getLedgerInfoByType(ledger_id)
+            ledger_info.set_defaults()
+            ledger_info.canSync = True
+            self.owner.request_ledger_status_from_nodes(ledger_id)
+            self.processStashedLedgerStatuses(ledger_id)
+        except KeyError:
+            logger.error("ledger type {} not present in ledgers so "
+                         "cannot set state".format(ledger_id))
+            return
+
+    def ledger_to_sync_after(self, ledger_id) -> Optional[int]:
+        if self.ledger_sync_order:
+            try:
+                idx = self.ledger_sync_order.index(ledger_id)
+                if idx < (len(self.ledger_sync_order) - 1):
+                    return self.ledger_sync_order[idx + 1]
+            except ValueError:
+                return None
+
     def getCatchupReqs(self, consProof: ConsistencyProof):
         # TODO: This needs to be optimised, there needs to be a minimum size
         # of catchup requests so if a node is trying to catchup only 50 txns
@@ -936,7 +992,7 @@ class LedgerManager(HasActionQueue):
                                 key=operator.itemgetter(0)))
 
     def getConsistencyProof(self, status: LedgerStatus):
-        ledger = self.getLedgerForMsg(status)    # type: Ledger
+        ledger = self.getLedgerForMsg(status)  # type: Ledger
         ledgerId = getattr(status, f.LEDGER_ID.nm)
         seqNoStart = getattr(status, f.TXN_SEQ_NO.nm)
         seqNoEnd = ledger.size
