@@ -9,7 +9,7 @@ from crypto.bls.bls_multi_signature import MultiSignatureValue
 from plenum.common.constants import VALID_LEDGER_IDS
 from plenum import PLUGIN_LEDGER_IDS
 from plenum.common.plenum_protocol_version import PlenumProtocolVersion
-from plenum.common.error import error
+from common.error import error
 from plenum.config import BLS_MULTI_SIG_LIMIT
 
 
@@ -102,6 +102,13 @@ class BooleanField(FieldBase):
         return
 
 
+class IntegerField(FieldBase):
+    _base_types = (int,)
+
+    def _specific_validation(self, val):
+        return
+
+
 class NonEmptyStringField(FieldBase):
     _base_types = (str,)
 
@@ -233,7 +240,7 @@ class NetworkPortField(FieldBase):
     _base_types = (int,)
 
     def _specific_validation(self, val):
-        if val < 0 or val > 65535:
+        if val <= 0 or val > 65535:
             return 'network port out of the range 0-65535'
 
 
@@ -283,11 +290,16 @@ class MessageField(FieldBase):
 
 class LedgerIdField(ChooseField):
     _base_types = (int,)
-    ledger_ids = VALID_LEDGER_IDS + tuple(PLUGIN_LEDGER_IDS)
+    ledger_ids = VALID_LEDGER_IDS
 
     def __init__(self, **kwargs):
+        self.ledger_ids = self.update_with_plugin_ledger_ids()
         super().__init__(self.ledger_ids, **kwargs)
 
+    @staticmethod
+    def update_with_plugin_ledger_ids():
+        return VALID_LEDGER_IDS + tuple(PLUGIN_LEDGER_IDS)
+
 
 class Base58Field(FieldBase):
     _base_types = (str,)
