@@ -1,5 +1,3 @@
-import asyncio
-import time
 from collections import deque
 from typing import List, Optional, Tuple
 
@@ -7,9 +5,10 @@ from stp_core.common.log import getlogger
 from stp_core.ratchet import Ratchet
 
 from plenum.common.throttler import Throttler
-from plenum.common.constants import PRIMARY_SELECTION_PREFIX, VIEW_CHANGE_PREFIX, MONITORING_PREFIX, POOL_LEDGER_ID
+from plenum.common.constants import PRIMARY_SELECTION_PREFIX, \
+    VIEW_CHANGE_PREFIX, MONITORING_PREFIX, POOL_LEDGER_ID
 from plenum.common.messages.node_messages import InstanceChange, ViewChangeDone
-from plenum.common.util import mostCommonElement, SortedDict, pop_keys
+from plenum.common.util import mostCommonElement, SortedDict
 from plenum.common.message_processor import MessageProcessor
 from plenum.server.models import InstanceChanges
 from plenum.server.has_action_queue import HasActionQueue
@@ -145,7 +144,7 @@ class ViewChanger(HasActionQueue, MessageProcessor):
     @property
     def has_view_change_from_primary(self) -> bool:
         if not self._has_view_change_from_primary:
-            next_primary_name = self.node.elector.next_primary_node_name(0)
+            next_primary_name = self.node.elector._next_primary_node_name_for_master()
 
             if next_primary_name not in self._view_change_done:
                 logger.debug(
@@ -525,7 +524,14 @@ class ViewChanger(HasActionQueue, MessageProcessor):
         if self.view_change_in_progress:
             self.view_change_in_progress = False
             self.node.on_view_change_complete()
-            self.instanceChanges.pop(self.view_no - 1, None)
+            # when we had INSTANCE_CHANGE message, they added into instanceChanges
+            # by msg.view_no. When view change was occured and view_no is changed,
+            # then we should delete all INSTANCE_CHANGE messages with current (already changed)
+            # view_no (which used in corresponded INSTANCE_CHANGE messages)
+            # Therefore we delete all INSTANCE_CHANGE messages from previous and current view number
+            for view_number in list(self.instanceChanges.keys()):
+                if view_number <= self.view_no:
+                    self.instanceChanges.pop(view_number, None)
             self.previous_master_primary = None
             self.propagate_primary = False
 
@@ -573,7 +579,7 @@ class ViewChanger(HasActionQueue, MessageProcessor):
         This method is called when sufficient number of ViewChangeDone
         received and makes steps to switch to the new primary
         """
-        expected_primary = self.node.elector.next_primary_node_name(0)
+        expected_primary = self.node.elector._next_primary_node_name_for_master()
         if new_primary != expected_primary:
             logger.error("{}{} expected next primary to be {}, but majority "
                          "declared {} instead for view {}"
@@ -589,7 +595,7 @@ class ViewChanger(HasActionQueue, MessageProcessor):
         """
         Sends ViewChangeDone message to other protocol participants
         """
-        new_primary_name = self.node.elector.next_primary_node_name(0)
+        new_primary_name = self.node.elector._next_primary_node_name_for_master()
         ledger_summary = self.node.ledger_summary
         message = ViewChangeDone(self.view_no,
                                  new_primary_name,
