@@ -1,7 +1,5 @@
 from plenum.server.quorums import Quorum
-from plenum.test.bls.helper import check_bls_multi_sig_after_send
-from plenum.test.pool_transactions.conftest import looper, clientAndWallet1, \
-    client1, wallet1, client1Connected
+from plenum.test.bls.helper import sdk_check_bls_multi_sig_after_send
 
 nodeCount = 7
 nodes_wth_bls = 4
@@ -14,10 +12,10 @@ def test_each_node_has_bls(txnPoolNodeSet):
 
 
 def test_send_txns_bls_less_than_consensus(looper, txnPoolNodeSet,
-                                           client1, client1Connected, wallet1):
+                                           sdk_pool_handle, sdk_wallet_client):
     # make sure that we have commits from all nodes, and have 4 of 7 ( < n-f) BLS sigs there is not enough
     for node in txnPoolNodeSet:
         node.quorums.commit = Quorum(nodeCount)
-    check_bls_multi_sig_after_send(looper, txnPoolNodeSet,
-                                   client1, wallet1,
-                                   saved_multi_sigs_count=0)
+    sdk_check_bls_multi_sig_after_send(looper, txnPoolNodeSet,
+                                       sdk_pool_handle, sdk_wallet_client,
+                                       saved_multi_sigs_count=0)
