@@ -11,9 +11,6 @@ from plenum.test.pool_transactions.helper import \
     disconnect_node_and_ensure_disconnected, \
     reconnect_node_and_ensure_connected
 
-# noinspection PyUnresolvedReferences
-from plenum.test.pool_transactions.conftest import \
-    clientAndWallet1, client1, wallet1, client1Connected, looper
 from stp_core.loop.eventually import eventually
 
 logger = getlogger()
@@ -21,12 +18,6 @@ txnCount = 5
 
 
 def make_master_replica_lag(node):
-
-    # class AbysmalBox(list):
-    #     def append(self, object) -> None:
-    #         pass
-    #
-    # node.replicas._master_replica.inBox = AbysmalBox()
     node.nodeIbStasher.delay(ppDelay(1200, 0))
     node.nodeIbStasher.delay(pDelay(1200, 0))
     node.nodeIbStasher.delay(cDelay(1200, 0))
@@ -50,11 +41,8 @@ def replicas_synced(node):
     assert compare_last_ordered_3pc(node) == 0
 
 
-def test_node_catchup_causes_no_desync(looper,
-                                       txnPoolNodeSet,
-                                       client1,
-                                       wallet1,
-                                       client1Connected):
+def test_node_catchup_causes_no_desync(looper, txnPoolNodeSet, client1,
+                                       wallet1, monkeypatch):
     """
     Checks that transactions received by catchup do not
     break performance monitoring
@@ -66,6 +54,9 @@ def test_node_catchup_causes_no_desync(looper,
 
     # Make master replica lagging by hiding all messages sent to it
     make_master_replica_lag(lagging_node)
+    monkeypatch.setattr(lagging_node.master_replica,
+                        '_request_missing_three_phase_messages',
+                        lambda *x, **y: None)
 
     # Send some requests and check that all replicas except master executed it
     sendReqsToNodesAndVerifySuffReplies(looper, wallet, client, 5)
@@ -78,9 +69,7 @@ def test_node_catchup_causes_no_desync(looper,
                                             txnPoolNodeSet,
                                             lagging_node,
                                             stopNode=False)
-    looper.removeProdable(lagging_node)
     sendReqsToNodesAndVerifySuffReplies(looper, wallet, client, 5)
-    looper.add(lagging_node)
     reconnect_node_and_ensure_connected(looper, txnPoolNodeSet, lagging_node)
 
     # Check that catchup done
