@@ -1,19 +1,20 @@
 # Start with 8 nodes
+from plenum.test.node_request.helper import sdk_ensure_pool_functional
+
 from plenum.common.constants import NYM, ROLE, STEWARD
 from plenum.common.constants import TXN_TYPE
 from plenum.test.helper import assertEquality
-from plenum.test.primary_selection.test_primary_selection_pool_txn import \
-    ensure_pool_functional
 
 nodeCount = 8
 
 
 def test_genesis_nodes(looper, txnPoolNodeSet,
-                       client1, wallet1, client1Connected):
+                       sdk_pool_handle,
+                       sdk_wallet_client):
     assert len(txnPoolNodeSet) == nodeCount
     for node in txnPoolNodeSet:
         assertEquality(node.poolLedger.size, nodeCount)
         stw_count = sum(1 for _, txn in node.domainLedger.getAllTxn() if
                         (txn[TXN_TYPE] == NYM) and (txn.get(ROLE) == STEWARD))
         assertEquality(stw_count, nodeCount)
-    ensure_pool_functional(looper, txnPoolNodeSet, wallet1, client1)
+    sdk_ensure_pool_functional(looper, txnPoolNodeSet, sdk_wallet_client, sdk_pool_handle)
