@@ -3,6 +3,6 @@ from plenum.test.conftest import looper
 nodeCount = 7
 
 
-def test_routes(nodeSet, up):
+def test_routes(txnPoolNodeSet):
     # TODO: Low priority.
     pass
