@@ -1,7 +1,5 @@
 from plenum.test.test_node import getNonPrimaryReplicas
 from plenum.test.view_change.helper import view_change_in_between_3pc
-from plenum.test.pool_transactions.conftest import clientAndWallet1, \
-    client1, wallet1, client1Connected, looper
 
 
 def slow_nodes(node_set):
@@ -9,7 +7,7 @@ def slow_nodes(node_set):
 
 
 def test_view_change_in_between_3pc_2_of_4_nodes_with_non_primary(
-        txnPoolNodeSet, looper, wallet1, client):
+        txnPoolNodeSet, looper, wallet1, client1):
     """
     - Slow processing 3PC messages for 2 of 4 node (2>f).
     - Both nodes are non-primary for master neither in this nor the next view
@@ -17,11 +15,11 @@ def test_view_change_in_between_3pc_2_of_4_nodes_with_non_primary(
     """
     view_change_in_between_3pc(looper, txnPoolNodeSet,
                                slow_nodes(txnPoolNodeSet),
-                               wallet1, client)
+                               wallet1, client1)
 
 
 def test_view_change_in_between_3pc_2_of_4_nodes_with_non_primary_long_delay(
-        txnPoolNodeSet, looper, wallet1, client):
+        txnPoolNodeSet, looper, wallet1, client1):
     """
     - Slow processing 3PC messages for 2 of 4 node (2>f).
     - Both nodes are non-primary for master neither in this nor the next view
@@ -29,5 +27,5 @@ def test_view_change_in_between_3pc_2_of_4_nodes_with_non_primary_long_delay(
     """
     view_change_in_between_3pc(looper, txnPoolNodeSet,
                                slow_nodes(txnPoolNodeSet),
-                               wallet1, client,
+                               wallet1, client1,
                                slow_delay=20)
