@@ -1,25 +1,23 @@
 from plenum.test.view_change.helper import view_change_in_between_3pc
-from plenum.test.pool_transactions.conftest import clientAndWallet1, \
-    client1, wallet1, client1Connected, looper
 
 
 def test_view_change_in_between_3pc_all_nodes(txnPoolNodeSet, looper,
-                                              wallet1, client):
+                                              wallet1, client1):
     """
     - Slow processing 3PC messages for all nodes
     - do view change
     """
     view_change_in_between_3pc(looper, txnPoolNodeSet, txnPoolNodeSet, wallet1,
-                               client)
+                               client1)
 
 
 def test_view_change_in_between_3pc_all_nodes_long_delay(
-        txnPoolNodeSet, looper, wallet1, client):
+        txnPoolNodeSet, looper, wallet1, client1):
     """
     - Slow processing 3PC messages for all nodes
     - do view change
     """
     view_change_in_between_3pc(looper, txnPoolNodeSet,
                                txnPoolNodeSet,
-                               wallet1, client,
+                               wallet1, client1,
                                slow_delay=20)
