@@ -24,13 +24,6 @@ MAX_RECONNECT_RETRY_ON_SAME_SOCKET = 1
 # Enables/disables debug mode for Looper class
 LOOPER_DEBUG = False
 
-# RAET Configuration
-RAETLogLevel = "terse"
-RAETLogLevelCli = "mute"
-RAETLogFilePath = os.path.join(os.path.expanduser(baseDir), "test.log")
-RAETLogFilePathCli = None
-RAETMessageTimeout = 60
-
 
 # Zeromq configuration
 DEFAULT_LISTENER_QUOTA = 100
